#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>
#include <X11/keysym.h>
#include <Imlib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <X11/X.h>
#include <X11/Xatom.h>
#include <signal.h>

//// TODO: Have pogo-remote make up a number for use as a key.
//// Send that key along with the directive request.
//// Have Pogo post that number after its done, so pogo-remote will know that it has succeeded in directing pogo.
//// Might be a fix for the flakiness.


int directivePusher(void);
void dumpUsage(void);
void dumpVersionInfo(void);
int randomValue(void);


char directiveType[20];
char configFileName[200];
char directiveFileName[200];
int primaryArgument=127;
int secondaryArgument=0;
extern char *optarg;
extern int optind;
int c;
FILE *fp=NULL;
int sleepCounter=0;


int main(int argc, char **argv)
{
	sprintf(configFileName,"(empty)");
	if(argc<=1) dumpUsage();

	sprintf(directiveType,"(empty)");
	fp=fopen("/tmp/.pogo-lock","r");

	while(!fp)
	{
		fp=fopen("/tmp/.pogo-lock","r");
		printf("Pogo-Remote: Uhoh...Pogo's lockfile is missing!. Is Pogo even running? Hmm. Sleeping 10 seconds...\n\n");
		sleepCounter++;
		if(sleepCounter==6)
		{
			printf("Pogo-Remote: Ugh. Giving up.\n");
			exit(1);
		}
		sleep(10);
	}

	fclose(fp);
                                                                                                                                                                       
        while ((c=getopt(argc,argv,"a:b:c:d:v"))!=-1) switch (c)
        {
                case 'a':
                primaryArgument=atoi(optarg);
                break;
                                                                                                                                                                       
                case 'b':
                secondaryArgument=atoi(optarg);
                break;
                                                                                                                                                                       
                case 'd':
                strcpy(directiveType,optarg);
                break;

		
		case 'c':
		strcpy(configFileName,optarg);
		break;


                case 'v':
		dumpVersionInfo();
                break;
                                                                                                                                                                       
                case '?':
		dumpUsage();
                break;
        }
                                                                                                                                                                       
	directivePusher();
	return 0;


}

void dumpUsage(void)
{
	printf("\n\n Usage:  pogo-remote -d <directive> -a [primaryArgument] -b [secondaryArgument] -c [filename]\n\n\n");
	printf(" %-20s%-20s%-20s%-20s\n ----------------------------------------------------------------------------------\n","Directive Name","Primary Argument","Secondary Argument","Directive String");
	printf(" %-20s%-20s%-20s%-20s\n","colorizeThisIcon","<unique ID>","<hue value>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","decolorizeThisIcon","<unique ID>","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","loadNewConfigFile","N/A","N/A","<config file name>");
	printf(" %-20s%-20s%-20s%-20s\n","slideThisIconUp","<icon number>","<spaces>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideThisIconDown","<icon number>","<spaces>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideThisIconLeft","<icon number>","<spaces>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideThisIconRight","<icon number>","<spaces>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","punchThisIcon","<unique ID>","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideAllIconsUp","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideAllIconsDown","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideAllIconsLeft","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","slideAllIconsRight","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","moveWindow","<x position>","<y position>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","strobeThisIcon","<unique ID>","<strobe count>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","throbThisIcon","<unique ID>","<strobe count>","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","switchToHorizontal","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","switchToVertical","N/A","N/A","N/A");
	printf(" %-20s%-20s%-20s%-20s\n","startThisProgram","<unique ID>","N/A","N/A\n\n");
	printf(" Example: pogo-remote -d punchThisIcon -a 4\n");
	printf(" Example: pogo-remote -d strobeThisIcon -a 3 -b 10\n");
	printf(" Example: pogo-remote -d loadNewConfigFile -c /home/foo/pogo.config\n\n");
	printf(" For further details regarding directive usage and options, please refer to the Pogo Directive README.\n\n");
	exit(0);
}


void dumpVersionInfo(void)
{
	printf("\nPogo-remote v2.2 by Bowie J. Poag  (Last updated Sep 24 1979)\n\n");
	printf("It's not safe to assume this version of pogo-remote will work with anything other than Pogo 2.2,\n");
	printf("as the signalling mechanism in Pogo may be written differently in subsequent releases.\n\n");
	exit(0);
}



int directivePusher (void)
{
	int ticket=0;
	fp=NULL;
	ticket=(randomValue());
	
	sprintf(directiveFileName,"/tmp/.pogo-directive.%d",ticket);

	while(!fp)
	{
		fp=fopen(directiveFileName,"ab");  
	}        


	fprintf(fp,"%s %i %i %s\n",directiveType,primaryArgument,secondaryArgument,configFileName);
        fclose(fp);
	return 0;
}



int randomValue (void)
{
        FILE *fp;
        unsigned char c1;
        srand(time(0));
                                                        
        if ((fp = fopen("/dev/urandom", "r")) == NULL)
        {
                fprintf(stderr, "Pogo-Remote: Error! Could not open /dev/urandom for read....WTF?\n");
                return -1;
        }
                                                                                                   
        c1 = fgetc(fp);
        fclose(fp);
        return c1%10;
}


