///
/// Pogo v2.2 Copyright (c) 1979, 1980 Bowie J. Poag <bpoag@comcast.net> 
/// Last updated 9/17/2006. 
///
/// Thanks: 
/// Jonathan Walther <krooger@debian.org> for ICCCM compliance code, and general guidance.
/// Ashley Winters <awinters@users.sourceforge.net> for the RGB hex triplet extractor code.
/// Carsten Haitzler <raster@rasterman.com> for the advice on solving the memory leak in 2.1
///
/// Usage:    pogo [-v verbosity_value] [-s icon_size] [-c configuration_filename] 
///		   [-n for no windowmanager decoration] [-q to show splash screen]
///
/// Example:  pogo -v1 -c mypogoconfig.txt
///	      pogo -n
///	      pogo -s32 -q
///
/// Legal stuff:
///
///    This program is free software; you can redistribute it and/or modify
///    it under the terms of the GNU General Public License as published by
///    the Free Software Foundation; either version 2 of the License, or
///    (at your option) any later version.
///
///    This program is distributed in the hope that it will be useful,
///    but WITHOUT ANY WARRANTY; without even the implied warranty of
///    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
///    GNU General Public License (http://www.gnu.org) for more details.
///
///    Coal in, ghost out.

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>
#include <X11/keysym.h>
#include <Imlib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <X11/X.h>
#include <X11/Xatom.h>
#include <signal.h>

void askWMForDelete(void);
void autoPilot(void);
void colorizeThisIcon(int, int);
void convertHueToRGB(int);
void createLockFile(void);
void decolorizeThisIcon(int);
void destroyStaleDirectives(void);
void destroyLockFile(void);
void drawIcons(void);
void drawThisIcon(int);
void explicitlyHighlightThisIcon(int);
void ghostThisIcon(int);
void highlightThisIcon(int);
void moveThisIconLeft(int);
void moveThisIconRight(int);
void parseConfigFile(void);
void parseDirectives(void);
void punchThisIcon(int);
void rotateAllIconsLeft(void);
void rotateAllIconsRight(void);
void rotateWindow(void);
void selectThisIcon(void);
void setIconColorInteractively(int);
void setUpTheWindow(void);
void setWindowDimensions(void);
void setWMHints(void);
void slideAllIconsDown(int);
void slideAllIconsLeft(int);
void slideAllIconsRight(int);
void slideAllIconsUp(int);
void slideThisIconDown(int);
void slideThisIconLeft(int);
void slideThisIconRight(int);
void slideThisIconUp(int);
void sparkleAllIcons(void);
void splashScreen(void);
void startThisProgram(int);
void strobeThisIcon(int,int);
void switchWindowOrientation(int);
void throbThisIcon(int,int);
void transportThisIcon(int);
void turnOffAllIconLights(void);
void turnThisIconOff(int);
void unghostThisIcon(int);

int randomValue(void);
int saveConfig(void);

Atom wmProtocols;
Atom wmDeleteWindow;
XWMHints *wmHints;
XSizeHints *sizeHints;
XClassHint *classHints;
XTextProperty wname;
GC gc;
KeySym key;
XEvent ev;
XSetWindowAttributes attr;
XWindowAttributes wa;
Window win;
Display *disp;
ImlibData *bufferData;
ImlibImage *buffer;
ImlibColorModifier mod;
int startingXPosition=0;
int startingYPosition=0;
int selectedIcon=127;
int totalIconWidth=0;
int totalIconHeight=0;
int totalIconCount=0;
int weveBeenHereBefore=0;
int flipFlop=0;
int windowOrientation=0; 
int screenWidth=0;
int screenHeight=0;
int noDecoration=0;
int iconSize=64;
int verboseMode=0;
int arguments=0;
int lastIconLit=127;
int red=0;
int green=0;
int blue=0;
int splashToggle=0;
int offsetMultiplier=2;
int lastIcon=127;
char configuration[200];
char newWindowTitle[100];
char originalWindowTitle[100];
double hue=1;
double intensity=0;


struct list_type 
{
	char filename[1024];
	char command[1024];
	char offline[1024];
	int ghostedState;
	int redValue;
	int greenValue;
	int blueValue;
	int panelStatus;
	int uniqueID;
	ImlibImage *image;
	ImlibData *imageData;
	ImlibColorModifier mod;

} icon[128];


int main(int argc, char **argv)
{
	sprintf(configuration,"/usr/local/pogo-2.2/configs/pogo.config");
	destroyLockFile();
	destroyStaleDirectives();
	createLockFile();
	XClientMessageEvent *cmsg;
	disp=XOpenDisplay(NULL);

	while ((arguments=getopt(argc,argv,"c:s:n:q:v"))!=-1) switch (arguments) 
	{
		case 's':
		iconSize=atoi(optarg);
		break;

		case 'n':
		noDecoration=1;
		break;

		case 'q':
		splashToggle=1;
		break;

		case 'v':
		verboseMode=1;
		break;
	
		case 'c':
		sprintf(configuration,"%s",optarg);
		break;

		case '?':
		exit(1);
		break;
	}
	
	parseConfigFile();
	if(splashToggle==1)splashScreen();
	setUpTheWindow();
	askWMForDelete();
	setWindowDimensions();
	drawIcons();



  	while(1)
	{
		//XSync(disp,False);


		while (!XPending(disp))
           	{
                	parseDirectives();
                	usleep(10000);
           	}

		XSync(disp,False);
        	XNextEvent(disp, &ev);

        	switch (ev.type) 
		{
        		case ButtonPress:

				if(ev.xbutton.button==1) 
				{       
					if(verboseMode==1)printf("Pogo: Mousebutton 1 pressed.\n"); 
					selectThisIcon();
					if(icon[selectedIcon].ghostedState==0)punchThisIcon(icon[selectedIcon].uniqueID);
					if(icon[selectedIcon].ghostedState==0)startThisProgram(icon[selectedIcon].uniqueID);
					break;
				}

				if(ev.xbutton.button==2)
				{
					if(verboseMode==1)printf("Pogo: Mousebutton 2 pressed.\n"); 
					selectThisIcon();
					setIconColorInteractively(icon[selectedIcon].uniqueID);
					break;
				}
		
				if(ev.xbutton.button==3)
				{
					
					if(verboseMode==1)printf("Pogo: Mousebutton 3 pressed.\n");
					selectThisIcon();
					transportThisIcon(selectedIcon);
					selectThisIcon();
					explicitlyHighlightThisIcon(selectedIcon);
                                        break;
				}

				if(ev.xbutton.button==4)
				{
					if(verboseMode==1)printf("Pogo: Mousebutton 4 pressed.\n"); 
					if(windowOrientation==0)slideAllIconsRight(1);
					if(windowOrientation==1)slideAllIconsUp(1);
					drawIcons();
					selectThisIcon();
					explicitlyHighlightThisIcon(selectedIcon);
                			if(icon[selectedIcon].ghostedState==1) sprintf(newWindowTitle,"Pogo: This icon is currently deactivated.");
                			if(icon[selectedIcon].ghostedState==0) sprintf(newWindowTitle,"Pogo: Click here to launch %s",icon[selectedIcon].command);
             				XStoreName(disp,win,newWindowTitle);
					break;
				}

				if(ev.xbutton.button==5)
				{
					if(verboseMode==1)printf("Pogo: Mousebutton 5 pressed.\n"); 
					if(windowOrientation==0)slideAllIconsLeft(1);
					if(windowOrientation==1)slideAllIconsDown(1);
					drawIcons();
					selectThisIcon();
					explicitlyHighlightThisIcon(selectedIcon);
                			if(icon[selectedIcon].ghostedState==1) sprintf(newWindowTitle,"Pogo: This icon is currently deactivated.");
                			if(icon[selectedIcon].ghostedState==0) sprintf(newWindowTitle,"Pogo: Click here to launch %s",icon[selectedIcon].command);
             				XStoreName(disp,win,newWindowTitle);
					break;
				}

				if(ev.xbutton.button==6)
				{
					if(verboseMode==1)printf("Pogo: Mousebutton 6 pressed.\n"); 
					saveConfig();
					break;
				}

				if(ev.xbutton.button==7)
				{
					if(verboseMode==1)printf("Pogo: Mousebutton 7 pressed.\n"); 
					flipFlop=windowOrientation;
                                        if(flipFlop==1) switchWindowOrientation(0);
                                        if(flipFlop==0) switchWindowOrientation(1);
					break;
				}
				break;


			case KeyPress:

                   		XLookupString(&ev.xkey, NULL, 0, &key, NULL);
                       		switch (key)
                       		{
                               		case XK_Up:
					if(verboseMode==1)printf("Pogo: Up key pressed\n");
					selectThisIcon();
                               		if(windowOrientation==0)slideThisIconRight(selectedIcon);	
                               		if(windowOrientation==1)slideAllIconsUp(1);	
					drawIcons();
					explicitlyHighlightThisIcon(selectedIcon);
                               		break;
                               		
					case XK_Down:
					if(verboseMode==1)printf("Pogo: Down key pressed\n");
					selectThisIcon();
                               		if(windowOrientation==0)slideThisIconLeft(selectedIcon);	
                               		if(windowOrientation==1)slideAllIconsDown(1);	
					drawIcons();
					explicitlyHighlightThisIcon(selectedIcon);
                               		break;
		
					case XK_Left:
					if(verboseMode==1)printf("Pogo: Left key pressed\n");
					selectThisIcon();
                                        if(windowOrientation==0)slideAllIconsLeft(1);
                                        if(windowOrientation==1)slideThisIconDown(selectedIcon);
					drawIcons();
					explicitlyHighlightThisIcon(selectedIcon);
					break;

					case XK_Right:
					if(verboseMode==1)printf("Pogo: Right key pressed\n");
					selectThisIcon();
                                        if(windowOrientation==0)slideAllIconsRight(1);
                                        if(windowOrientation==1)slideThisIconUp(selectedIcon);
					drawIcons();
					explicitlyHighlightThisIcon(selectedIcon);
					break;

					case XK_Tab:
                                        if(verboseMode==1)printf("Pogo: Tab key pressed. Flipping orientation..\n");
                                        flipFlop=windowOrientation;
                                        if(flipFlop==1) switchWindowOrientation(0);
                                        if(flipFlop==0) switchWindowOrientation(1);
                                        break;

					case XK_c:
					if(verboseMode==1)printf("Pogo: Right key pressed\n");
					selectThisIcon();
					throbThisIcon(icon[selectedIcon].uniqueID,1);
					setIconColorInteractively(icon[selectedIcon].uniqueID);
					throbThisIcon(icon[selectedIcon].uniqueID,1);
					break;

					case XK_Return:
					if(verboseMode==1)printf("Pogo: Return pressed. Launching selected icon..\n");
					selectThisIcon();
                                        punchThisIcon(icon[selectedIcon].uniqueID);
                                        startThisProgram(icon[selectedIcon].uniqueID);
					break;

					case XK_Escape:
					destroyLockFile();
					return 0;	
					break;

					case XK_BackSpace:
					if(verboseMode==1)printf("Pogo: Icon colorization strip request recieved. Stripping color..\n");
					selectThisIcon();
					decolorizeThisIcon(icon[selectedIcon].uniqueID);
					break;

					case XK_space:
					if(verboseMode==1)printf("Pogo: Spacebar pressed\n");
					saveConfig();
					break;
                       		}
				break;

			case VisibilityNotify:
				
				if(verboseMode==1)printf("Pogo: VisibilityNotify event recieved.\n");
				drawIcons();
				break;

			
			case MotionNotify:

				if(verboseMode==1)printf("Pogo: MotionNotify event recieved.\n");
				selectThisIcon();
				if(verboseMode==1)printf("Pogo: Pointer is at: %d,%d Icon currently being pointed to: %d (UniqueID is %d)\n",ev.xbutton.x, ev.xbutton.y, selectedIcon, icon[selectedIcon].uniqueID);
				highlightThisIcon(selectedIcon);
                                break;

			case EnterNotify:

				if(verboseMode==1)printf("Pogo: EnterNotify event recieved.\n");
				selectThisIcon();
				explicitlyHighlightThisIcon(selectedIcon);
                                break;

			case LeaveNotify:

				if(verboseMode==1)printf("Pogo: LeaveNotify event recieved.\n");
				turnOffAllIconLights();
				drawIcons();
				sprintf(originalWindowTitle,"Pogo v2.2 by Bowie J. Poag");
        			XStoreName(disp,win,originalWindowTitle);
				XSync(disp,False);
				autoPilot();
				break;
				
			case Expose:
				
				if(verboseMode==1)printf("Pogo: Expose event recieved.\n");
				//drawIcons();
				//XSync(disp,True);
				break;

			//case GraphicsExpose:

			//	if(verboseMode==1)printf("Pogo: GraphicsExpose event recieved.\n");
			//	drawIcons();
			//	XSync(disp,False);
			//	break;

			case ClientMessage:

				if(verboseMode==1)printf("Pogo: ClientMessage event recieved.\n");
	                        cmsg = &ev.xclient;
	                        if (cmsg->message_type == wmProtocols && cmsg->data.l[0] == wmDeleteWindow) 
				{
					destroyLockFile();
					return 0;
				}
				break;

			case MapNotify:

				if(verboseMode==1)printf("Pogo: MapNotify event recieved.\n");
				drawIcons();
				XSync(disp,False);
				break;
			
		}

	printf("Pogo: Before, X's Queue is %d deep.\n",XEventsQueued(disp,QueuedAlready));
	XSync(disp,False);
	printf("Pogo: After, X's Queue is %d deep.\n",XEventsQueued(disp,QueuedAlready));
	}


}

void decolorizeThisIcon(int uniqueID)
{
   	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside decolorizeThisIcon()..\n");

	while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                                                                                                                                                                                  
                iconNumber++;
        }

	Imlib_get_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
	mod.brightness=255;
	mod.gamma=255;
	Imlib_set_image_red_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
        Imlib_set_image_green_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
        Imlib_set_image_blue_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
        Imlib_set_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);

	icon[iconNumber].redValue=255;
	icon[iconNumber].greenValue=255;
	icon[iconNumber].blueValue=255;

	buffer=icon[iconNumber].image;
	Imlib_get_image_modifier(bufferData,buffer,&mod);
	mod.brightness=255;
	mod.gamma=255;
	Imlib_set_image_red_modifier(bufferData,buffer,&mod);
	Imlib_set_image_green_modifier(bufferData,buffer,&mod);
	Imlib_set_image_blue_modifier(bufferData,buffer,&mod);
	Imlib_set_image_modifier(bufferData,buffer,&mod);
	if(windowOrientation==0) Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
	if(windowOrientation==1) Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
}


void autoPilot(void)
{

	if(verboseMode==1) printf("Pogo: Elvis has left the building. Switching to autoPilot() mode...\n");

	while (1)
	{
		parseDirectives();
		usleep(20000);
		//if(XCheckTypedWindowEvent(disp,win,VisibilityNotify,&ev)) drawIcons();
		if(XCheckTypedWindowEvent(disp,win,Expose,&ev)) drawIcons();
		if(XCheckTypedWindowEvent(disp,win,EnterNotify,&ev)) return;
		XSync(disp,True);
		
	}
}
	


void switchWindowOrientation(int orientation)
{
	if(orientation==0) 
	{
		windowOrientation=0;
		setWindowDimensions();
		XSync(disp,False);
		drawIcons();
		explicitlyHighlightThisIcon(selectedIcon);
		selectedIcon=127;
		XSync(disp,False);
		return;
	}

	if(orientation==1) 
	{
		windowOrientation=1;
		setWindowDimensions();
		XSync(disp,False);
		drawIcons();
		explicitlyHighlightThisIcon(selectedIcon);
		selectedIcon=127;
		XSync(disp,False);
		return;
	}

}


void setIconColorInteractively(int uniqueID)
{
	int lastX=ev.xbutton.x;
	int lastY=ev.xbutton.y;
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside setIconColorInteractively()...\n");
	if(verboseMode==1) printf("Pogo: Attempting to set the color of Unique iD %d interactively.",uniqueID);

	highlightThisIcon(127);

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }


	drawIcons();

	while(1)
	{

		XNextEvent(disp, &ev);

		if(ev.type == ButtonRelease && ev.xbutton.button==2) return;

		if(ev.type==ButtonPress || ev.type==KeyPress || ev.type==MotionNotify)
		{
			XLookupString(&ev.xkey, NULL, 0, &key, NULL);
                        switch (key)
                        {
                                case XK_Up:
                                hue+=3;
                                break;
                                                                                                                                                                   
                                case XK_Down:
                                hue-=3;
                                break;

				case XK_c:
				return;

				case XK_Left:
				intensity=intensity-3;
				break;
			
				case XK_Right:
				intensity=intensity+3;
				break;
                        }


			if(ev.xbutton.y>lastY)
			{ 
				intensity=intensity-5;
			}

			if(ev.xbutton.y<lastY)
			{
				intensity=intensity+5;
			}
 
			if(ev.xbutton.x>lastX) hue++;
			if(ev.xbutton.x<lastX) hue--;
			if(hue<1) hue=1;
			if(hue>359) hue=359;
		
			lastX=ev.xbutton.x;
			lastY=ev.xbutton.y;
			
			convertHueToRGB(hue);
						
        		Imlib_get_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
               		mod.gamma=red;
               		Imlib_set_image_red_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
               		mod.gamma=green;
               		Imlib_set_image_green_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
               		mod.gamma=blue;
               		Imlib_set_image_blue_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);

			buffer=icon[iconNumber].image;
              
			Imlib_get_image_modifier(bufferData,buffer,&mod);
			icon[iconNumber].redValue=red;
               		mod.gamma=red;
               		Imlib_set_image_red_modifier(bufferData,buffer,&mod);
			icon[iconNumber].greenValue=green;
               		mod.gamma=green;
               		Imlib_set_image_green_modifier(bufferData,buffer,&mod);
			icon[iconNumber].blueValue=blue;
               		mod.gamma=blue;
               		Imlib_set_image_blue_modifier(bufferData,buffer,&mod);

               		if(windowOrientation==0) Imlib_paste_image(bufferData,buffer,win,1+(iconSize*lastIconLit),1,iconSize,iconSize);
               		if(windowOrientation==1) Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*lastIconLit),iconSize,iconSize);
		
			drawThisIcon(iconNumber);
		}
	} 
} 


void slideAllIconsUp(int moveCount)
{
        int x=0;
        int offset=1;

	if(windowOrientation==0) return;

	turnOffAllIconLights();

	while(moveCount!=0)
	{
		if(verboseMode==1)printf("Pogo: Sliding all icons up..\n");	
        	XGetWindowAttributes(disp,win,&wa);
        	
		while(offset<=iconSize)
        	{
                	for(x=0;x<=totalIconCount;x++)
                	{
                       	 	Imlib_paste_image(icon[x].imageData,icon[x].image,win,1,-offset+(1+iconSize*x),iconSize,iconSize);
                        	Imlib_paste_image(icon[0].imageData,icon[0].image,win,1,totalIconCount*iconSize-(offset/2),iconSize,iconSize);
                        	XDrawRectangle(disp,win,gc,0,0,wa.width-1,wa.height);
                        	XSync(disp,False);
                	}

                	x=0;
                	usleep(1);
                	XSync(disp,False);
                	offset*=offsetMultiplier;
        	}
        	rotateAllIconsLeft();
        	drawIcons();
		moveCount--;
	}
	selectThisIcon();
	highlightThisIcon(selectedIcon);
}


void slideAllIconsLeft(int moveCount)
{
        int x=0;
        int offset=1;
                                                                                                                                                                                 
	turnOffAllIconLights();
	
	if(windowOrientation==1) return;
	while(moveCount!=0)
	{
		if(verboseMode==1)printf("Pogo: Sliding all icons to the left..\n");	
        	XGetWindowAttributes(disp,win,&wa);
                                                                                                                                                                                  
        	while(offset<=iconSize)
        	{
                	for(x=0;x<totalIconCount;x++)
                	{
                        	Imlib_paste_image(icon[x].imageData,icon[x].image,win,-offset+(1+iconSize*x),1,iconSize,iconSize);
                        	Imlib_paste_image(icon[0].imageData,icon[0].image,win,(totalIconCount)*iconSize-(offset)+1,1,iconSize,iconSize);
                        	XDrawRectangle(disp,win,gc,0,0,wa.width-1,wa.height);
				XSync(disp,False);
                	}
                	x=0;
                	usleep(1);
                	XSync(disp,False);
                	offset*=offsetMultiplier;
        	}
        	rotateAllIconsLeft();
        	drawIcons();
		moveCount--;
	}

	selectThisIcon();
	highlightThisIcon(selectedIcon);
}


void slideAllIconsDown(int moveCount)
{
        int x=0;
        int offset=1;
	
	turnOffAllIconLights();
	
if(windowOrientation==0) return;

	while(moveCount!=0)
	{
		if(verboseMode==1)printf("Pogo: Sliding all icons down..\n");	
        	XGetWindowAttributes(disp,win,&wa);
        	
		while(offset<=iconSize)
        	{
                	for(x=0;x<=totalIconCount;x++)
                	{
                        	Imlib_paste_image(icon[x].imageData,icon[x].image,win,1,offset+(1+iconSize*x),iconSize,iconSize);
                        	Imlib_paste_image(icon[totalIconCount-1].imageData,icon[totalIconCount-1].image,win,1,1+(-(iconSize-offset)),iconSize,iconSize);
                        	XDrawRectangle(disp,win,gc,0,0,wa.width-1,wa.height);
                        	XSync(disp,False);
                	}
			x=0;
                	usleep(1);
                	XSync(disp,False);
                	offset*=offsetMultiplier;
        	}
        	rotateAllIconsRight();
        	drawIcons();
		moveCount--;
	}
	selectThisIcon();
	highlightThisIcon(selectedIcon);
}


void slideThisIconUp(int uniqueID)
{
	int offset=1;
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside slideThisIconUp()..\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }

	
	if(iconNumber>=totalIconCount || iconNumber <0)return;
        if(verboseMode==1)printf("Pogo: Sliding icon #%d up..\n",iconNumber);
        while(offset<=iconSize)
        {
                if(iconNumber==0) return;  // Not a valid move.
                
		if(offset!=1)XFillRectangle(disp,win,gc,1, iconSize*(iconNumber-1),iconSize,iconSize*2);
                Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,1,iconSize*iconNumber-offset,iconSize,iconSize);
                Imlib_paste_image(icon[iconNumber-1].imageData,icon[iconNumber-1].image,win,1,offset+(1+iconSize*(iconNumber-1)),iconSize,iconSize);
                
		usleep(1);
                XSync(disp,False);
                offset*=offsetMultiplier;
        }
        moveThisIconRight(iconNumber-1);
        rotateAllIconsRight();
}


void slideThisIconDown(int uniqueID)
{
	int offset=1;
	int iconNumber=0;


	if(verboseMode==1) printf("Pogo: Inside slideThisIconDown()..\n");
        
        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }
	
	if(iconNumber==totalIconCount-1)
        {
                if(verboseMode==1) printf("Pogo: Request to slide icon beyond physical extent of window. That's a no-no. Returning to caller.\n");
                unghostThisIcon(iconNumber);
                return;
        }


	if(verboseMode==1)printf("Pogo: Sliding icon #%d down..\n",iconNumber);
       
	while(offset<=iconSize)
        {
                if(iconNumber==totalIconCount-1) return;  // Not a valid move.
                if(offset!=1)XFillRectangle(disp,win,gc,1, iconSize*iconNumber,iconSize,iconSize*2);
                Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,1,offset+(1+iconSize*iconNumber+1),iconSize,iconSize);
                Imlib_paste_image(icon[iconNumber+1].imageData,icon[iconNumber+1].image,win,1,-offset+(1+iconSize*(iconNumber+1)),iconSize,iconSize);

                usleep(1);
                XSync(disp,False);
                offset*=offsetMultiplier;
        }
        moveThisIconLeft(iconNumber+1);
        rotateAllIconsLeft();

}


void selectThisIcon(void)
{
	if(windowOrientation==0)selectedIcon=ev.xbutton.x/iconSize;
	if(windowOrientation==1)selectedIcon=ev.xbutton.y/iconSize;
	if(windowOrientation==0 && (ev.xbutton.x <=0 || ev.xbutton.x >= totalIconCount*iconSize || ev.xbutton.y >= totalIconHeight || ev.xbutton.y <=0 )) selectedIcon=127;
	if(windowOrientation==1 && (ev.xbutton.y <=0 || ev.xbutton.y >= totalIconCount*iconSize || ev.xbutton.x <= 0 || ev.xbutton.x >= totalIconWidth )) selectedIcon=127;
	if(verboseMode==1)printf("Pogo: The icon currently being pointed to: %d\n",selectedIcon);
}

void slideThisIconRight(int uniqueID)
{
        int offset=1;
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside slideThisIconRight()...\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }

	if(iconNumber==totalIconCount-1)
	{
		if(verboseMode==1) printf("Pogo: Request to slide icon beyond physical extent of window. That's a no-no. Returning to caller.\n");
		unghostThisIcon(iconNumber);
		return;
	}

        if(verboseMode==1)printf("Pogo: Sliding icon #%d to the right, totalIconCount is %d..\n",iconNumber,totalIconCount);

	while(offset<=iconSize)
	{
		if(iconNumber>=totalIconCount-1 || iconNumber <0)return;

		if(offset!=1)XFillRectangle(disp,win,gc,iconSize*(iconNumber),1,iconSize*2,iconSize);
		Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,offset+(1+iconSize*iconNumber+1),1,iconSize,iconSize);
		Imlib_paste_image(icon[iconNumber+1].imageData,icon[iconNumber+1].image,win,-offset+(1+iconSize*(iconNumber+1)),1,iconSize,iconSize);

                usleep(1);
                XSync(disp,False);
                offset*=offsetMultiplier;
	}
	moveThisIconLeft(iconNumber+1);
	rotateAllIconsLeft();
}

void slideThisIconLeft(int uniqueID)
{
	int offset=1;
	int iconNumber=0;
	
	if(verboseMode==1) printf("Pogo: Inside slideThisIconLeft()...\n");
        
        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }


	if(verboseMode==1)printf("Pogo: Sliding icon #%d to the right, totalIconCount is %d..\n",iconNumber,totalIconCount);

        while(offset<=iconSize)
        {
                if(iconNumber==0) return;  // Not a valid move.
		
		if(offset!=1)XFillRectangle(disp,win,gc,iconSize*(iconNumber-1),1,iconSize*2,iconSize);
                Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,iconSize*iconNumber-offset,1,iconSize,iconSize);
		Imlib_paste_image(icon[iconNumber-1].imageData,icon[iconNumber-1].image,win,offset+(1+iconSize*(iconNumber-1)),1,iconSize,iconSize);
                XSync(disp,False);
		usleep(1);
                offset*=offsetMultiplier;
        }
        moveThisIconRight(iconNumber-1);
        rotateAllIconsRight();
}


void slideAllIconsRight(int moveCount)
{
	int x=0;
	int offset=1;
	
	turnOffAllIconLights();
	if(windowOrientation==1) return;

	while(moveCount!=0)
	{
		if(verboseMode==1)printf("Pogo: Sliding all icons to the right..\n");	
		XGetWindowAttributes(disp,win,&wa);

		while(offset<=iconSize)
		{
			for(x=0;x<=totalIconCount;x++)
			{
				Imlib_paste_image(icon[x].imageData,icon[x].image,win,offset+(1+iconSize*x),1,iconSize,iconSize);
				Imlib_paste_image(icon[totalIconCount-1].imageData,icon[totalIconCount-1].image,win,1+(-(iconSize-offset)),1,iconSize,iconSize);
				XDrawRectangle(disp,win,gc,0,0,wa.width-1,wa.height);	
				XSync(disp,False);
			}

			x=0;
			usleep(1);
			XSync(disp,False);
			offset*=offsetMultiplier;
		}
		rotateAllIconsRight();
		drawIcons();
		moveCount--;
	}
	selectThisIcon();
	highlightThisIcon(selectedIcon);
}

void transportThisIcon(int iconNumber)
{
	int xpos=1;
	int ypos=1;
	int grabOffset=0;
	int targetLocation=0;

	if(windowOrientation==0) grabOffset=ev.xbutton.x%iconSize;
	if(windowOrientation==1) grabOffset=ev.xbutton.y%iconSize;

	while(ev.type!=ButtonRelease)
	{

		if(ev.type==MotionNotify)
		{
			if(windowOrientation==0)
			{
					xpos=ev.xbutton.x;
					if(((xpos-grabOffset)/iconSize)-3>=0) drawThisIcon(((xpos-grabOffset)/iconSize)-3);
					if(((xpos-grabOffset)/iconSize)-2>=0) drawThisIcon(((xpos-grabOffset)/iconSize)-2);
					if(((xpos-grabOffset)/iconSize)-1>=0) drawThisIcon(((xpos-grabOffset)/iconSize)-1);
					if((xpos-grabOffset)/iconSize>=0) drawThisIcon(((xpos-grabOffset)/iconSize));
					if(((xpos-grabOffset)/iconSize)+1<=totalIconCount) drawThisIcon(((xpos-grabOffset)/iconSize)+1);
					if(((xpos-grabOffset)/iconSize)+2<=totalIconCount) drawThisIcon(((xpos-grabOffset)/iconSize)+2);
					if(((xpos-grabOffset)/iconSize)+3<=totalIconCount) drawThisIcon(((xpos-grabOffset)/iconSize)+3);
					if(xpos-grabOffset<=0) xpos=grabOffset;
					if(xpos-grabOffset>=iconSize*(totalIconCount-1)) xpos=iconSize*(totalIconCount-1)+grabOffset;
					Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,xpos-grabOffset,1,iconSize,iconSize); 
					XSync(disp,False);
			}

			if(windowOrientation==1)
			{
					ypos=ev.xbutton.y;
                        		if(((ypos-grabOffset)/iconSize)-3>=0) drawThisIcon(((ypos-grabOffset)/iconSize)-3);
                        		if(((ypos-grabOffset)/iconSize)-2>=0) drawThisIcon(((ypos-grabOffset)/iconSize)-2);
                        		if(((ypos-grabOffset)/iconSize)-1>=0) drawThisIcon(((ypos-grabOffset)/iconSize)-1);
                        		if((ypos-grabOffset)/iconSize>=0) drawThisIcon(((ypos-grabOffset)/iconSize));
                        		if(((ypos-grabOffset)/iconSize)+1<=totalIconCount) drawThisIcon(((ypos-grabOffset)/iconSize)+1);
                        		if(((ypos-grabOffset)/iconSize)+2<=totalIconCount) drawThisIcon(((ypos-grabOffset)/iconSize)+2);
                        		if(((ypos-grabOffset)/iconSize)+3<=totalIconCount) drawThisIcon(((ypos-grabOffset)/iconSize)+3);
					if(ypos-grabOffset<=0) ypos=grabOffset;
                                        if(ypos-grabOffset>=iconSize*(totalIconCount-1)) ypos=iconSize*(totalIconCount-1)+grabOffset;
                        		Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image,win,1,ypos-grabOffset,iconSize,iconSize);
                        		XSync(disp,False);
			}
		}
		XNextEvent(disp,&ev);
	}
	if(windowOrientation==0)targetLocation=ev.xbutton.x/iconSize;
	if(windowOrientation==1)targetLocation=ev.xbutton.y/iconSize;
	if(verboseMode==1) printf("Pogo: Processing move request. Target location: %d  Start location: %d\n",targetLocation,iconNumber);

	if(targetLocation==iconNumber)
	{
		if(verboseMode==1) printf("Pogo: Starting and Target locations are equal -- Stripping off any icon colorization.\n");
                decolorizeThisIcon(icon[selectedIcon].uniqueID);
		drawThisIcon(iconNumber);
		if(iconNumber-1>=0) drawThisIcon(iconNumber-1);
		if(iconNumber+1<=totalIconCount) drawThisIcon(iconNumber+1);
		XSync(disp,False);
		return;
	}
	

	if(targetLocation<0)
	{
		if(verboseMode==1) printf("Pogo: User has overdragged the icon. Setting target location to 0.\n");
		targetLocation=0; 
	}

	if(targetLocation>totalIconCount-1)
	{
		if(verboseMode==1) printf("Pogo: User has overdragged the icon. Setting target location to %d.",totalIconCount-1);
		targetLocation=totalIconCount-1;
	}


	highlightThisIcon(127);
	ghostThisIcon(icon[iconNumber].uniqueID);

	if(windowOrientation==0)
	{
		if(targetLocation<iconNumber)
		{
			while(targetLocation!=iconNumber && targetLocation>=0 && targetLocation<=totalIconCount)
			{
				if(verboseMode==1) printf("Pogo: Sliding launcher %d to position %d.\n",icon[iconNumber].uniqueID,iconNumber);
				slideThisIconLeft(icon[iconNumber].uniqueID);
				iconNumber--;
			}
		}
	

		if(targetLocation>iconNumber)
        	{
			while(targetLocation!=iconNumber && targetLocation>=0 && targetLocation<=totalIconCount)
                	{
				if(verboseMode==1) printf("Pogo: Sliding launcher %d to position %d.\n",icon[iconNumber].uniqueID,iconNumber);
                       	 	slideThisIconRight(icon[iconNumber].uniqueID);
                        	iconNumber++;
                	}
        	}
	}


	if(windowOrientation==1)
	{
  		if(targetLocation<iconNumber)
                {
			while(targetLocation!=iconNumber && targetLocation>=0 && targetLocation<=totalIconCount)
                        {
				if(verboseMode==1) printf("Pogo: Sliding launcher %d to position %d.\n",icon[iconNumber].uniqueID,iconNumber);
                                slideThisIconUp(icon[iconNumber].uniqueID);
                                iconNumber--;
                        }
                }
                                                                                                                                                               
                if(targetLocation>iconNumber)
                {
			ghostThisIcon(icon[iconNumber].uniqueID);
			while(targetLocation!=iconNumber && targetLocation>=0 && targetLocation<=totalIconCount)
                        {
				if(verboseMode==1) printf("Pogo: Sliding launcher %d to position %d.\n",icon[iconNumber].uniqueID,iconNumber);
                                slideThisIconDown(icon[iconNumber].uniqueID);
                                iconNumber++;
                        }
                }
	}


	unghostThisIcon(icon[iconNumber].uniqueID);
}


void ghostThisIcon(int uniqueID)
{

	int iconNumber=0;

        if(verboseMode==1) printf("Pogo: Inside ghostThisIcon()...\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }


	if(verboseMode==1) printf("Pogo: Dimming this icon, probably as part of a move operation.\n");

	icon[iconNumber].ghostedState=1;

        Imlib_get_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
        while(icon[iconNumber].mod.brightness>=180)
	{
		usleep(1);
	        Imlib_set_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
		drawThisIcon(iconNumber);
		icon[iconNumber].mod.brightness-=10;
	}
}


void unghostThisIcon(int uniqueID)
{

	int iconNumber=0;

        if(verboseMode==1) printf("Pogo: Inside unghostThisIcon()...\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }


	if(verboseMode==1) printf("Pogo: Un-dimming this icon, probably as part of a move operation.\n");
	icon[iconNumber].ghostedState=0;

        Imlib_get_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
        while(icon[iconNumber].mod.brightness<=255)
	{
		usleep(1);
        	Imlib_set_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
		drawThisIcon(iconNumber);
		icon[iconNumber].mod.brightness+=10;
	}
}


void drawThisIcon(int iconNumber)
{
	if(verboseMode==1)printf("Pogo: Recieved explicit draw request for Icon %d.\n",iconNumber);
	if(iconNumber<0 || iconNumber >totalIconCount-1)
	{
		if(verboseMode==1) printf("Pogo: Draw request is outside physical window extent. Returning to caller.\n");
		return;
	}

		if(windowOrientation==0)Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image, win, 1+(iconSize*iconNumber), 1, iconSize, iconSize);
		if(windowOrientation==1)Imlib_paste_image(icon[iconNumber].imageData,icon[iconNumber].image, win, 1, 1+(iconSize*iconNumber), iconSize, iconSize);
		XSync(disp,False);

}


void drawIcons(void)
{
	int x=0;

	if(verboseMode==1) printf("Pogo: Inside drawIcons()..\n");
	

	if(windowOrientation==0)
	{
		while(x<totalIconCount)
		{	
			if(icon[x].panelStatus==1)
			{
				mod.gamma=512;
				mod.brightness=275;
				mod.contrast=300;
				Imlib_set_image_modifier(icon[x].imageData,icon[x].image,&mod);
			}


			if(icon[x].panelStatus==0)
			{
				mod.brightness=255;
				mod.gamma=255;
				mod.contrast=255;
				Imlib_set_image_modifier(icon[x].imageData,icon[x].image,&mod);
			}

			Imlib_paste_image(icon[x].imageData,icon[x].image, win, 1+(iconSize*x), 1, iconSize, iconSize);
			x++;
		}
	}
                                                                                                                                                                             
        if(windowOrientation==1)
        {
                while(x<totalIconCount)
                {

		  	if(icon[x].panelStatus==1)
                        {
                                mod.brightness=275;
                                mod.gamma=512;
                                mod.contrast=300;
                                Imlib_set_image_modifier(icon[x].imageData,icon[x].image,&mod);
                        }

                        if(icon[x].panelStatus==0)
                        {
                                mod.brightness=255;
                                mod.gamma=255;
                                mod.contrast=255;
                                Imlib_set_image_modifier(icon[x].imageData,icon[x].image,&mod);
                        }

                       	Imlib_paste_image(icon[x].imageData,icon[x].image, win, 1, 1+(iconSize*x), iconSize, iconSize);
                        x++;
                }
        }
	XSync(disp,False);
	XFlush(disp);		
}


void moveThisIconRight(int iconNumber)
{
	icon[127]=icon[iconNumber+1];
	icon[iconNumber+1]=icon[iconNumber];
	icon[iconNumber]=icon[127];
	rotateAllIconsLeft();
}


void createLockFile(void)
{
	FILE *fp;

	if ((fp = fopen("/tmp/.pogo-lock", "w")) == NULL)  fprintf(stderr, "\nUhoh. Pogo cant create it's lockfile. (/tmp/.pogo.lock) ..Fatal.\n"); 
                                                                                                                                                                       
        fprintf(fp,"Pogo!");
        fclose(fp);
	return;

}

void destroyLockFile(void)
{
	system("rm -f /tmp/.pogo-lock");
	return;
}


void destroyStaleDirectives(void)
{
	system("rm -f /tmp/.pogo-directive*");
	return;
}


void moveThisIconLeft(int iconNumber)
{
	icon[127]=icon[iconNumber-1];
	icon[iconNumber-1]=icon[iconNumber];
	icon[iconNumber]=icon[127];
	rotateAllIconsRight();
}


void rotateAllIconsLeft(void)
{
	int x=0;
	
	icon[127]=icon[0];

	while(x<totalIconCount)
	{
		icon[x]=icon[x+1];
		x++;
	}
	
	icon[totalIconCount-1]=icon[127];
}


void rotateAllIconsRight(void)
{
	int x=totalIconCount-1;
	
	icon[127]=icon[totalIconCount-1];

        while(x>0)
        {
		icon[x]=icon[x-1];
		x--;
        }
        
	icon[0]=icon[127];
}


void splashScreen(void)
{
	screenWidth=XDisplayWidth(disp,0);
	screenHeight=XDisplayHeight(disp,0);
        attr.background_pixel=0;
        bufferData=Imlib_init(disp);
        buffer=Imlib_load_image(bufferData,"/usr/local/pogo-2.2/images/pogo-logo.jpg");
        win=XCreateWindow(disp,DefaultRootWindow(disp), screenWidth/2-(buffer->rgb_width/2), screenHeight/2-(buffer->rgb_height/2), buffer->rgb_width,buffer->rgb_height,0,CopyFromParent,InputOutput,CopyFromParent,CWBackPixel,&attr);
	setWMHints();
	sprintf(originalWindowTitle,"Pogo v2.2 by Bowie J. Poag");
        XStoreName(disp,win,originalWindowTitle);
        XMapWindow(disp,win);
	
	Imlib_get_image_modifier(bufferData,buffer,&mod);
	mod.brightness=450;
	while(mod.brightness>255)
	{
		Imlib_apply_image(bufferData,buffer,win);
		mod.brightness-=2.5;
		Imlib_set_image_modifier(bufferData,buffer,&mod);
		XSync(disp,False);
	}

	sleep(1);
	
	while(mod.brightness<512)
        {
                Imlib_apply_image(bufferData,buffer,win);
                mod.brightness+=2.5;
		Imlib_set_image_modifier(bufferData,buffer,&mod);
		XSync(disp,False);
        }

	mod.brightness=255;
	Imlib_set_image_modifier(bufferData,buffer,&mod);
	XDestroyWindow(disp,win);
}


void strobeThisIcon(int uniqueID, int strobeCycles)
{
        int originalBrightness;
        int iconNumber=0;

        if(verboseMode==1) printf("Pogo: Inside strobeThisIcon()...\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }

        buffer=icon[iconNumber].image;
        Imlib_get_image_modifier(bufferData,buffer,&mod);
        originalBrightness=mod.brightness;

        while(strobeCycles!=0)
        {
                        while(mod.brightness<320)
                        {
                                Imlib_set_image_modifier(bufferData,buffer,&mod);
                                if(windowOrientation==0)Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
                                if(windowOrientation==1)Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
                                usleep(1000);
                                mod.brightness=mod.brightness+75;
                        }


                        while(mod.brightness>=originalBrightness)
                        {
                                Imlib_set_image_modifier(bufferData,buffer,&mod);
                                if(windowOrientation==0)Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
                                if(windowOrientation==1)Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
                                usleep(1000);
                                mod.brightness=mod.brightness-75;
                        }

                strobeCycles--;
        }

        XSync(disp,False);

}


void throbThisIcon(int uniqueID,int throbCycles)
{
	int originalBrightness;
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside throbThisIcon()...\n");

	while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                iconNumber++;
        }

	buffer=icon[iconNumber].image;
	Imlib_get_image_modifier(bufferData,buffer,&mod);
	originalBrightness=mod.brightness;

	while(throbCycles!=0)
	{
			while(mod.brightness<320)
			{
				Imlib_set_image_modifier(bufferData,buffer,&mod);
                                if(windowOrientation==0)Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
                                if(windowOrientation==1)Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
                                usleep(9000);
                                mod.brightness=mod.brightness+25;
			}

			
			while(mod.brightness>=originalBrightness)
			{
				Imlib_set_image_modifier(bufferData,buffer,&mod);
                        	if(windowOrientation==0)Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
                        	if(windowOrientation==1)Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
				usleep(9000);
				mod.brightness=mod.brightness-5;
			}

		throbCycles--;
	}

	XSync(disp,False);
}


void setUpTheWindow(void)
{
        screenWidth=XDisplayWidth(disp,0);
        screenHeight=XDisplayHeight(disp,0);
	attr.backing_store=Always;
	attr.background_pixel=BlackPixel(disp,DefaultScreen(disp));
	if(noDecoration==1) attr.override_redirect=True;
     	bufferData=Imlib_init(disp);
        buffer=Imlib_load_image(bufferData,icon[0].filename);
        gc = DefaultGC(disp,DefaultScreen(disp));
	if(verboseMode==1)printf("Pogo: Attempting to draw window at %d,%d\n",startingXPosition,startingYPosition);
    	win=XCreateWindow(disp,DefaultRootWindow(disp),startingXPosition,startingYPosition,buffer->rgb_width,buffer->rgb_height,0,CopyFromParent,InputOutput,CopyFromParent,CWBackPixel | CWBackingStore | CWOverrideRedirect,&attr);
	setWMHints();
	sprintf(originalWindowTitle,"Pogo v2.2 by Bowie J. Poag");
        XStoreName(disp,win,originalWindowTitle);
        XSelectInput(disp, win, ButtonPressMask|ButtonReleaseMask|VisibilityChangeMask|ExposureMask|StructureNotifyMask|KeyPressMask|EnterWindowMask|LeaveWindowMask|SubstructureNotifyMask|PointerMotionMask);
        XMapWindow(disp,win);
	XSync(disp,False);
}
                                                                                                                      
int saveConfig (void)
{
        FILE *fp;
        int x=0;
                                                                                                                      
        if((fp=fopen(configuration,"w"))==NULL)
        {
                fprintf(stderr, "\nError! Can't open config file (pogo.config) for some reason. Hmm.\n");
                return -1;
        }

	XGetWindowAttributes(disp,win,&wa);
	if(verboseMode==1)printf("\nPogo: Saving config file. Orientation/Position line will be %d %d %d\n",windowOrientation,wa.x,wa.y);
	fprintf(fp,"%d %d %d\n",windowOrientation,wa.x,wa.y);

        while(x<totalIconCount)
        {
                fprintf(fp, "%-4d%-40s %d %d %d %-6s\n",icon[x].uniqueID,icon[x].filename,icon[x].redValue,icon[x].greenValue,icon[x].blueValue,icon[x].command);
		if(verboseMode==1)printf("Pogo: Writing..... %-4d%-40s %d %d %d %-6s\n",icon[x].uniqueID,icon[x].filename,icon[x].redValue,icon[x].greenValue,icon[x].blueValue,icon[x].command);
                x++;
        }
        fclose(fp);
	sparkleAllIcons();
return 0;
}

void sparkleAllIcons(void)
{
	int x=0;

	if(verboseMode==1)printf("Pogo: Performing save notification sparkle..\n");
        while(x<totalIconCount)
        {
                highlightThisIcon(x);
                x++;
                usleep(1);
        }
        x=totalIconCount;
        while(x>=0)
        {
                highlightThisIcon(x);
                x--;
                usleep(1);
        }
	
	highlightThisIcon(127);

}

void parseConfigFile(void)
{
	FILE *configFile;
	FILE *existenceCheck;
	int x=0;
	int orientation=0;
	
        if ((configFile = fopen(configuration, "r")) == NULL)
        {
                fprintf(stderr, "\nPogo:  Could not locate or open the config file (%s).. Giving up.\n\n",configuration);
                exit(1);
        }

	if(verboseMode==1) printf("Pogo: Inside parseConfigFile..\n");
	fscanf(configFile,"%d %d %d[^\n]",&orientation,&startingXPosition,&startingYPosition);
	if(orientation==0) windowOrientation=0;
	if(orientation==1) windowOrientation=1;


	while(fscanf(configFile, "%d %s %d %d %d %[^\n]",&icon[x].uniqueID,icon[x].filename,&icon[x].redValue,&icon[x].greenValue,&icon[x].blueValue,icon[x].command)!=EOF)
	{
		if((existenceCheck = fopen(icon[x].filename,"r"))==NULL)
		{
			fprintf(stderr, "\nPogo: A launcher defined in one of Pogo's configuration files (Unique ID #%d, inside %s maybe..?) is munged. This is a fatal error. Go have a look at the file, and straighten it out. :)\n\n ",icon[x].uniqueID, configuration);
			exit(1);	
		}
		fclose(existenceCheck);
		if(weveBeenHereBefore)
		{
			Imlib_kill_image(icon[x].imageData,icon[x].image);
		}
		icon[x].imageData=Imlib_init(disp);
		icon[x].image=Imlib_load_image(icon[x].imageData,icon[x].filename);
		icon[x].panelStatus=0;
		totalIconWidth+=icon[x].image->rgb_width;
		if(icon[x].image->rgb_height>totalIconHeight) totalIconHeight=icon[x].image->rgb_height;
		Imlib_get_image_modifier(icon[x].imageData,icon[x].image,&mod);
		mod.gamma=icon[x].redValue;
		Imlib_set_image_red_modifier(icon[x].imageData,icon[x].image,&mod);
		mod.gamma=icon[x].greenValue;
		Imlib_set_image_green_modifier(icon[x].imageData,icon[x].image,&mod);
		mod.gamma=icon[x].blueValue;
		Imlib_set_image_blue_modifier(icon[x].imageData,icon[x].image,&mod);
		x++;
	}
	mod.gamma=255;
	totalIconCount=x;
        fclose(configFile);
	weveBeenHereBefore++;
}


void punchThisIcon(int uniqueID)
{
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside punchThisIcon()...\n");

	while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
                }
                                                                                                                                                                                  
                iconNumber++;
        }


        if(icon[iconNumber].ghostedState==1) return; 

	buffer=icon[iconNumber].image;
	Imlib_get_image_modifier(bufferData,buffer,&mod);
	mod.brightness=255;

	while(mod.brightness!=100)
	{
		Imlib_set_image_modifier(bufferData,buffer,&mod);
         	if(windowOrientation==0) Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
		if(windowOrientation==1) Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
		mod.brightness--;
	}

      while(mod.brightness<256)
        {
                Imlib_set_image_modifier(bufferData,buffer,&mod);
                if(windowOrientation==0) Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
                if(windowOrientation==1) Imlib_paste_image(bufferData,buffer,win,1,1+(iconSize*iconNumber),iconSize,iconSize);
                mod.brightness+=2;
        }
}

void explicitlyHighlightThisIcon(int uniqueID)
{
	int iconNumber=0;

	if(verboseMode==1) printf("Pogo: Inside explicitlyHighlightThisIcon()...\n");	

	while(icon[iconNumber].uniqueID!=uniqueID)
        {
                if(iconNumber>=totalIconCount)
                {
                        if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
                        return;
               }
                iconNumber++;
        }

	highlightThisIcon(iconNumber);

}


void highlightThisIcon(int iconNumber)
{
 	if(lastIcon!=iconNumber) 
	{
		turnOffAllIconLights();
		drawIcons();
		icon[lastIcon].panelStatus=0;
	}
	icon[iconNumber].panelStatus=1;
	drawIcons();
	lastIcon=iconNumber;
        if(iconNumber!=127)sprintf(newWindowTitle,"Pogo: Click here to launch %s",icon[iconNumber].command);
        XStoreName(disp,win,newWindowTitle);

}


void turnOffAllIconLights(void)
{
	int x=0;

        while(x<totalIconCount)
        {
                icon[x].panelStatus=0;
                x++;
        }

	drawIcons();

}

                               
void startThisProgram(int uniqueID)
{
   	int iconNumber=0;
	char command[4096];
                                                                                                                                                                                                                          
        while(icon[iconNumber].uniqueID!=uniqueID) iconNumber++; 

	sprintf(command, "%s &",icon[iconNumber].command);
	system(command);
}


void setWMHints(void)
{
        sizeHints = XAllocSizeHints();
        wmHints = XAllocWMHints();
        classHints = XAllocClassHint();
	sizeHints->x=startingXPosition;
	sizeHints->y=startingYPosition;
        sizeHints->width = totalIconCount*iconSize;
	sizeHints->min_width = iconSize;
	sizeHints->max_width = totalIconCount*iconSize;
        sizeHints->height = iconSize;
	sizeHints->min_height = iconSize;
	sizeHints->max_height = iconSize*totalIconCount;
        sizeHints->flags = USSize|PPosition|PSize|PMinSize|PMaxSize;
        wmHints->input = True;
        wmHints->initial_state = NormalState;
        wmHints->flags = StateHint | InputHint;
        classHints->res_name = "Pogo";
        classHints->res_class = "Pogo";
        XSetWMProperties(disp,win, NULL, NULL, NULL, 0, sizeHints, wmHints, classHints);
}

                                                                                                                                                                                  
void parseDirectives(void)
{
        FILE *directiveFile;
	char directiveString[100];
	char thisDirective[200];
	char directiveType[100];
	int  ticket;
	int  directiveArgument1;
	int  directiveArgument2;
	int rgb=0;
	int found=0;
	int directiveCounter=0;


	ticket=(randomValue());
	sprintf(thisDirective,"/tmp/.pogo-directive.%d",ticket);
	
        if ((directiveFile = fopen(thisDirective, "rw")) == NULL)
        {
		return;
        }
	

	while(fscanf(directiveFile, "%s %i %i %[^\n]",directiveType,&directiveArgument1,&directiveArgument2,directiveString)!=EOF)
	{
		directiveCounter++;

		if(verboseMode==1)
		{
			printf("Pogo: Found ticket #%d -- Processing directive (%s %i %i %s)\n",ticket,directiveType,directiveArgument1,directiveArgument2,directiveString);	
		}

		if(directiveArgument1<0 || directiveArgument2<0)
		{
			if(verboseMode==1) printf("Pogo: Invalid primary or secondary directive argument (Pri: %d) (Sec: %d).. Returning to caller.\n",directiveArgument1,directiveArgument2);
			return;
		}


		if(!strcmp(directiveType,"loadNewConfigFile"))
		{
			found=1;
			sprintf(configuration,"%s",directiveString);
			if(verboseMode==1) printf("Pogo: Recieved loadNewConfigFile directive. Configfile about to be loaded is %s.\n",configuration);
			parseConfigFile();
			setWindowDimensions();
			XSync(disp,False);
			drawIcons();
		}

		if(directiveArgument2<0 || directiveArgument1<0)
		{	
			drawIcons();
 			return;
		}

		if(!strcmp(directiveType,"decolorizeThisIcon")) { found=1; decolorizeThisIcon(directiveArgument1); }
		if(!strcmp(directiveType,"slideAllIconsUp")) { found=1; slideAllIconsUp(directiveArgument1); }
		if(!strcmp(directiveType,"slideAllIconsDown")) { found=1; slideAllIconsDown(directiveArgument1); }
		if(!strcmp(directiveType,"slideAllIconsRight")) { found=1; slideAllIconsRight(directiveArgument1); }
		if(!strcmp(directiveType,"slideAllIconsLeft")) { found=1; slideAllIconsLeft(directiveArgument1); }
		if(!strcmp(directiveType,"switchToHorizontal")) { found=1; switchWindowOrientation(0); }
		if(!strcmp(directiveType,"switchToVertical")) { found=1; switchWindowOrientation(1); }
		if(!strcmp(directiveType,"punchThisIcon")) { found=1; punchThisIcon(directiveArgument1); }
		if(!strcmp(directiveType,"startThisProgram")) { found=1; startThisProgram(directiveArgument1); }
		if(!strcmp(directiveType,"throbThisIcon")) { found=1; throbThisIcon(directiveArgument1,directiveArgument2); }
		if(!strcmp(directiveType,"strobeThisIcon")) { found=1; strobeThisIcon(directiveArgument1,directiveArgument2); }
		if(!strcmp(directiveType,"moveWindow")) { found=1; XMoveWindow(disp,win,directiveArgument1,directiveArgument2); }
		if(!strcmp(directiveType,"setIconSize")) 
		{
			found=1;
			iconSize=directiveArgument1; 
			setWindowDimensions();
			drawIcons();
		}
	
		if(!strcmp(directiveType,"colorizeThisIcon"))
        	{
			found=1;
        		rgb=strtol(directiveString,NULL,16);
        		colorizeThisIcon(directiveArgument1,rgb);
        	}

		if(!strcmp(directiveType,"slideThisIconLeft"))
		{
			found=1;
			while(directiveArgument2!=0)
			{
				slideThisIconLeft(directiveArgument1);
				directiveArgument2--;
			}
		}


		if(!strcmp(directiveType,"slideThisIconRight"))
        	{
			found=1;
               		while(directiveArgument2!=0)
               		{
                       		slideThisIconRight(directiveArgument1);
                       		directiveArgument2--;
               		}
        	}
                                                                                                                                                                                                                        
		if(!strcmp(directiveType,"slideThisIconUp"))
        	{
			found=1;
               		while(directiveArgument2!=0)
               		{
                       		slideThisIconUp(directiveArgument1);
                       		directiveArgument2--;
               		}
        	}
                                                                                                                                                                                                                        
		if(!strcmp(directiveType,"slideThisIconDown"))
        	{
			found=1;
               		while(directiveArgument2!=0)
               		{
                       		slideThisIconDown(directiveArgument1);
                       		directiveArgument2--;
               		}
        	}

		if(!found && verboseMode==1) printf("Pogo: The given directive is either wrong, or not supported in this version of Pogo.\n");
	}


        fclose(directiveFile);
	unlink(thisDirective);
	if(verboseMode==1 && found==1) printf("Pogo: Finished processing directive(s) from ticket %d.\n",ticket);
	return;
}

                                                                                                                                                                                  
void colorizeThisIcon(int uniqueID, int rgb)
{
	int iconNumber=0;
                                                                                                                                                                                                                          
	if(verboseMode==1) printf("Pogo: Inside colorizeThisIcon()..\n");

        while(icon[iconNumber].uniqueID!=uniqueID)
	{
		if(iconNumber>=totalIconCount)
		{
			if(verboseMode==1) printf("Pogo: Can't do anything..Requested UniqueID not found! Returning to caller.\n");
			return;
		}

		iconNumber++;
	}

	printf("Located uniqueID (%d) inside icon #%d\n",uniqueID,iconNumber);
        red=(rgb>>16)&0xff;
        green=(rgb>>8)&0xff;
        blue=rgb&0xff;

	buffer=icon[iconNumber].image;
                                                                                                                                               
	Imlib_get_image_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
	mod.gamma=red;
	Imlib_set_image_red_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
	mod.gamma=green;
	Imlib_set_image_green_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
	mod.gamma=blue;
	Imlib_set_image_blue_modifier(icon[iconNumber].imageData,icon[iconNumber].image,&icon[iconNumber].mod);
                                                                                                                                               
	buffer=icon[iconNumber].image;
                                                                                                                                               
	Imlib_get_image_modifier(bufferData,buffer,&mod);
	icon[iconNumber].redValue=red;
	mod.gamma=red;
	Imlib_set_image_red_modifier(bufferData,buffer,&mod);
	icon[iconNumber].greenValue=green;
	mod.gamma=green;
	Imlib_set_image_green_modifier(bufferData,buffer,&mod);
	icon[iconNumber].blueValue=blue;
	mod.gamma=blue;
	Imlib_set_image_blue_modifier(bufferData,buffer,&mod);

	Imlib_paste_image(bufferData,buffer,win,1+(iconSize*iconNumber),1,iconSize,iconSize);
	drawIcons();
}


void askWMForDelete(void)
{
	int status;
	
	wmProtocols = XInternAtom(disp, "WM_PROTOCOLS", False);
	wmDeleteWindow = XInternAtom(disp, "WM_DELETE_WINDOW", False);
	status = XSetWMProtocols(disp, win, & wmDeleteWindow, 1);
	if (status != True) printf("Pogo: could not ask for a clean delete for some reason.. Hmm.\n");
}


void convertHueToRGB(int hue)
{
                                                                                                                                                                                               
                if(hue<1) hue=1;
                if(hue>359) hue=359;

		if(red>500) red=500;
		if(green>500) green=500;
		if(blue>500) blue=500;
                                                                                                                                                                                               
                if(hue>=0 && hue<=59)
                {
                        red=255;
                        green=(255/60)*hue;
                        blue=0;
                }
                                                                                                                                                                                               
                if(hue>=60 && hue<=119)
                {
                        red=255-(255/60)*(hue-60);
                        green=255;
                        blue=0;
                }
                                                                                                                                                                                               
                if(hue>=120 && hue<=179)
                {
                        red=0;
                        green=255;
                        blue=(255/60)*(hue-120);
                }
                                                                                                                                                                                               
                if(hue>=180 && hue<=239)
                {
                        red=0;
                        green=255-(255/60)*(hue-180);
                        blue=255;
                }
                                                                                                                                                                                               
                if(hue>=240 && hue<=299)
                {
                        red=(255/60)*(hue-240);
                        green=0;
                        blue=255;
                }
                                                                                                                                                                                               
                if(hue>=300 && hue<=359)
                {
                        red=255;
                        green=0;
                        blue=255-(255/60)*(hue-300);
                }

		if(intensity>200) intensity=200;
		if(intensity<-200) intensity=-200;
		mod.brightness=255+intensity;

		red=red+intensity;
		green=green+intensity;
		blue=blue+intensity;
}


int randomValue (void)
{
        FILE *fp;
        unsigned char c1;
        srand(time(0));
                                                        
        if ((fp = fopen("/dev/urandom", "r")) == NULL)
        {
                fprintf(stderr, "\nError! Could not open /dev/urandom for read\n");
                return -1;
        }
                                                                                                   
        c1 = fgetc(fp);
        fclose(fp);
        return c1%10;
}


void setWindowDimensions(void)
{
        if(windowOrientation==0)
	{
		sizeHints->width = (totalIconCount*iconSize)+2;
        	sizeHints->min_width = (totalIconCount*iconSize)+2;
        	sizeHints->max_width = (totalIconCount*iconSize)+2;
        	sizeHints->height = iconSize+2;
        	sizeHints->min_height = iconSize+2;
        	sizeHints->max_height = iconSize+2;
	}

	if(windowOrientation==1)
	{
		sizeHints->width = iconSize+2;
                sizeHints->min_width = iconSize+2;
                sizeHints->max_width = iconSize+2;
                sizeHints->height = (totalIconCount*iconSize)+2;
                sizeHints->min_height = (totalIconCount*iconSize)+2;
                sizeHints->max_height = (totalIconCount*iconSize)+2;
	}

       	XSetWMProperties(disp,win, NULL, NULL, NULL, 0, sizeHints, wmHints, classHints);
	if(windowOrientation==0)XResizeWindow(disp,win,(iconSize*totalIconCount)+2,iconSize+2);
	if(windowOrientation==1)XResizeWindow(disp,win,iconSize+2,(iconSize*totalIconCount)+2);
}

